/*    */

#include <GL/glx.h>
#include <GL/glu.h>
#include <X11/keysym.h>
#include "../include/GL/glpng.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#ifndef EXIT_FAILURE
#  define EXIT_FAILURE    1
#endif
#ifndef EXIT_SUCCESS
#  define EXIT_SUCCESS    0
#endif

#define MAXQUAD 6

typedef float Vertex[3];

typedef Vertex Quad[4];

/* data to define the six faces of a unit cube */
Quad quads[MAXQUAD] = {
   { {0,0,0}, {1,0,0}, {1,1,0}, {0,1,0} },
   { {0,0,1}, {1,0,1}, {1,1,1}, {0,1,1} },
   { {0,0,0}, {1,0,0}, {1,0,1}, {0,0,1} },
   { {0,1,0}, {1,1,0}, {1,1,1}, {0,1,1} },
   { {0,0,0}, {0,0,1}, {0,1,1}, {0,1,0} },
   { {1,0,0}, {1,0,1}, {1,1,1}, {1,1,0} }
    };

static void error(const char* prog, const char* msg);
static void process_input(Display *dpy, Window win);
static int query_extension(char* extName);
#if 1
static int attributeList[] = { GLX_RGBA, GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1,
    GLX_BLUE_SIZE, 1, GLX_DOUBLEBUFFER, GLX_DEPTH_SIZE, 1, None };
#else
static int attributeList[] = { GLX_RGBA, GLX_RED_SIZE, 5, GLX_GREEN_SIZE, 6,
    GLX_BLUE_SIZE, 5, GLX_DOUBLEBUFFER, GLX_DEPTH_SIZE, 1, None };
#endif
static int dimension = 3;


static void draw_scene(int mx, int my) {
	int x, y, z, i;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glPushMatrix();

	/* track the mouse */
	glRotatef(mx / 2.0, 0, 1, 0);
	glRotatef(my / 2.0, 1, 0, 0);
	/* draw the lines as hidden polygons */
	glTranslatef(-0.5, -0.5, -0.5);
	for (i = 0; i < MAXQUAD; i++) {
		glColor3f(0, 0, 0);
		/* draw a filled polygon */
		glBegin(GL_QUADS);
		glTexCoord2f(1, 1);  glVertex3fv(quads[i][0]);
		glTexCoord2f(1, 0);  glVertex3fv(quads[i][1]);
		glTexCoord2f(0, 0);  glVertex3fv(quads[i][2]);
		glTexCoord2f(0, 1);  glVertex3fv(quads[i][3]);
		glEnd();
		}
	glPopMatrix();
	}


static void process_input(Display *dpy, Window win) {
    XEvent event;
    static int prevx, prevy;
    static int deltax = 90, deltay = 40;

    do {
	char buf[31];
	KeySym keysym;

	XNextEvent(dpy, &event);
	switch(event.type) {
	case Expose:
	    break;
	case ConfigureNotify: {
	    /* this approach preserves a 1:1 viewport aspect ratio */
	    int vX, vY, vW, vH;
	    int eW = event.xconfigure.width, eH = event.xconfigure.height;
	    if (eW >= eH) {
		vX = 0;
		vY = (eH - eW) >> 1;
		vW = vH = eW;
	    } else {
		vX = (eW - eH) >> 1;
		vY = 0;
		vW = vH = eH;
	    }
	    glViewport(vX, vY, vW, vH);
	    }
			break;
		  case KeyPress:
			(void) XLookupString(&event.xkey, buf, sizeof(buf), &keysym, NULL);
			switch (keysym) {
			  case XK_Escape:
				exit(EXIT_SUCCESS);
			  default:
				break;
				}
		  case ButtonPress:
			prevx = event.xbutton.x;
			prevy = event.xbutton.y;
			break;
		  case MotionNotify:
			deltax += (event.xbutton.x - prevx); prevx = event.xbutton.x;
			deltay += (event.xbutton.y - prevy); prevy = event.xbutton.y;
			break;
		  default:
			break;
			}
		} while (XPending(dpy));
	draw_scene(deltax, deltay);
	glXSwapBuffers(dpy, win);
	}


static void error(const char *prog, const char *msg) {
	fprintf(stderr, "%s: %s\n", prog, msg);
	exit(EXIT_FAILURE);
	}


int main(int argc, char** argv) {
	Display *dpy;
	XVisualInfo *vi;
	XSetWindowAttributes swa;
	Window win;
	GLXContext cx;
	GLuint texture;
	pngInfo info;
	char texfilename[128];
	int i;

	texfilename[0] = 0;
	for (i = 1; i < argc; i++)
		strcpy(texfilename, argv[i]);
	if (texfilename[0] == 0) {
		printf("Syntax:\n", argv[0]);
		printf("  glpngtest [PNG texture filename]\n", argv[0]);
		exit(EXIT_FAILURE);
		}
	dpy = XOpenDisplay(0);
	if (!dpy)
		error(argv[0], "can't open display");
	vi = glXChooseVisual(dpy, DefaultScreen(dpy), attributeList);
	if (!vi)
		error(argv[0], "no suitable visual");
	cx = glXCreateContext(dpy, vi, 0, GL_TRUE);
	swa.colormap = XCreateColormap(dpy, RootWindow(dpy, vi->screen),
	  vi->visual, AllocNone);
	swa.border_pixel = 0;
	swa.event_mask = ExposureMask | StructureNotifyMask | KeyPressMask |
	  ButtonPressMask | ButtonMotionMask;
	/*  0, 0, 300, 300  */
	win = XCreateWindow(dpy, RootWindow(dpy, vi->screen), 0, 0, 300, 300,
	  0, vi->depth, InputOutput, vi->visual,
	  CWBorderPixel|CWColormap|CWEventMask, &swa);
	XStoreName(dpy, win, "glpng test");
	XMapWindow(dpy, win);
	glXMakeCurrent(dpy, win, cx);
	/* set up viewing parameters */
	glMatrixMode(GL_PROJECTION);
/*	gluPerspective(20, 1, 0.1, 20);
/*	glLoadIdentity();  */
#define thang 0.015625
	glFrustum(-thang, thang, -thang, thang, 0.1, 20);
	glMatrixMode(GL_MODELVIEW);
	glTranslatef(0, 0, -5);  /*  0, 0, -15  */
	/* set other relevant state information */
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_FLAT);
	glDisable(GL_DITHER);  /*  16bpp looks better with this enabled  */

	/*  load default texture  */
#if 0
	texture = pngBind("Stunt.png", PNG_NOMIPMAP, PNG_SOLID, &info, GL_CLAMP, GL_NEAREST, GL_NEAREST);
#else
	texture = pngBind(texfilename, PNG_NOMIPMAP, PNG_SOLID, &info, GL_CLAMP, GL_LINEAR, GL_LINEAR);
#endif
	if (texture == 0) {
		puts("Can't load file");
		exit(1);
		}
	printf("Size=%i,%i Depth=%i Alpha=%i\n", info.Width, info.Height, info.Depth, info.Alpha);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	/* process events until the user presses ESC */
	while (1)
		process_input(dpy, win);
	return (0);
	}

